#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
$Id: wash.py,v 1.2 2014/06/06 11:52:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Программа перегонки сахарной браги

"""

import actuators, sqlite3, time, DS18B20db, Power
from actuators import *

#############Параметры колонны#####################################
#Температура срабатывания клапана холодильника
Tcond=30.0

#Температуры куба, которым соответствует мощность нагрева
#TempPower=[(83.5,0),(82,30),(81,25),(79,20),(65,15),(0,100)]
TempPower=[(84.0,0),(65.0,30),(0.0,100)]

#Соответствия установки температуры дефлегматора мощности
PowerTemp=[(20,63.5),(25,63.5),(30,63.5)]

#Температура дефлегматора отбора тела
Tdeph=PowerTemp[0][1]

#Таймаут ожидания завершения измерения (сек)
timeout=10

#Места и порядок расположения термометров:
locations=[u"Куб",u"1/2",u"Верх",u"Деф",u"Конд"]

##################################################################

def WaitBoiling(Tlist):
    """Ожидание условия закипания"""
    #Cоединение с базой данных
    cnct=sqlite3.connect("Distiller.db")
    #Создание курсора БД
    crsr=cnct.cursor()
    Vmax=0.0            #Скорость изменения температуры
    IDmax=u""           #ID термометра с макимальным ростом температуры
    while Vmax<1:       #Пока скорость изменения температуры менее 1град в сек
        #Нужно подождать результаты измерения температуры
        DS18B20db.ReadyData.wait()
        for ID in Tlist:    #Для каждого термометра из списка
            #Читаем два последних значения
            execute=u"SELECT ["+ID+u"] FROM DS18B20 ORDER BY DateTime DESC LIMIT 2"
            crsr.execute(execute)
            Ts=crsr.fetchall()
            if len(Ts)<2:
                break
            Tup=float(Ts[0][0]) #Самое свежее значение температуры
            Tlo=float(Ts[1][0]) #Предыдущее значение
            if (Tup-Tlo)>Vmax:  #Если разность превышает максимальну скорость
                Vmax=Tup-Tlo    #Новое значение скорости
                IDmax=ID        #Термометр
                #print(u"ID=%s, Vmax=%s"%(IDmax,Vmax))
        printT()
    crsr.execute(u"SELECT DateTime FROM DS18B20 ORDER BY DateTime DESC LIMIT 1")
    return (IDmax,Vmax,crsr.fetchone()[0])

def printT():
    """Вывод показаний термометров"""
    #Чтение термометров из БД
    Temp=""
    for dt in locations:
        db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
        Temp+=(u"%s=%3.1f" % (dt,round(float(db_cur.fetchone()[0]),1)))+"  "
    Temp+=("P=%s"%Power.P)
    print(Temp)

#Отключение всех реле
OffAll()
print("Отключение всех реле")

#Продувка
print("Заполнение холодильников")
OnDeph()
OnCond()
time.sleep(2)
OffAll()

#Cоединение с базой данных
db_con=sqlite3.connect("Distiller.db")
#Создание курсора БД
db_cur=db_con.cursor()
#Проверка списка термометров
db_cur.execute("SELECT * FROM DS18B20cur")
Tlist=[]     #Список термометров
for i in db_cur.fetchall():
    ID=i[0]
    Tlist.append(ID)
if len(Tlist)==0:
    End=raw_input("Цифровые термометры DS18B20 не обнаружены!!!")
    exit()    

#Запуск процесса измерения температур
DS18B20db.StartMeasure()
#Запуск записи температур в БД
DS18B20db.StartWrite()

#Установка полной мощности
Power.Set(100)
#Запуск преобразования Брезенхема
Power.On()
print("Мощность 100%")

#Ждем нагрева до закипания (как правило, термометр 1/2)
print("Ожидание закипания")
Boiling=WaitBoiling(Tlist)
#print(u"%s Vmax=%s %s"%Boiling)
Power.Set(20)
print("Разгон завершен")


print("Запуск регулирования процесса дистилляции")
while True:
    #Нужно подождать завершение измерения температуры
    timeB=time.time()
    DS18B20db.ReadyData.wait(timeout)
    if (time.time()-timeB) >= timeout:
        print("!!!ОШИБКА!!!: Сбой процесса измерения температур.")
        break

    #Чтение термометров из БД
    T=[]
    for dt in locations:
        db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
        T.append(float(db_cur.fetchone()[0]))

    #Если температура на выходе холодильника-конденсатора>Tcond,
    #включаем охлаждение. Иначе отключаем.
    if T[4]>Tcond:
        OnCond()
    else:
        OffCond()

    #Если температура на выходе дефлегматора>Tbody,
    #включаем охлаждение. Иначе отключаем.
    if T[3]>Tdeph:
        OnDeph()
    else:
        OffDeph()

    #Подаем мощность, соответствующую температуре в кубе
    P=0
    for stage in TempPower:
        if T[0]>stage[0]:
            P=stage[1]
            break
    Power.Set(P)

    #Устанавливаем температуру дефлегматора, соответствующую мощности
    for PT in PowerTemp:
        if PT[0]==P: Tdeph=PT[1]

    #Вывод показаний
    printT()

    #Проверка завершения процесса
    if P==0:
        AutoEn=False
        break

print("Процесс дистилляции завершен")
#Отключаем нарев
Power.Off()
print("Регулятор мощности останолен")

#Продувка
OnCond()
OnDeph()
time.sleep(1)

#Отключаем всё
OffAll()
print("Все реле отключены")
#Останавливаем запись температур в БД
DS18B20db.StopWrite()
DS18B20db.StopMeasure()

print("Процесс сбора температур остановлен")

