#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
$Id: CrudeAlcohol.py,v 1.2 2014/06/17 19:18:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Программа перегонки спирта-сырца

"""

import actuators, sqlite3, time, DS18B20db, Power
import threading
from actuators import *
import datetime
from dateutil.relativedelta import relativedelta

#Отключение всех реле
OffAll()
print("Отключение всех реле")

#Cоединение с базой данных
db_con=sqlite3.connect("Distiller.db")
#Создание курсора БД
db_cur=db_con.cursor()
#Проверка наличия термометров
db_cur.execute("SELECT * FROM DS18B20cur WHERE Name='Куб'")
if db_cur.fetchone()==None:
    End=raw_input("Необходима калибровка!!!")
    exit()

#Продувка
print("Заполнение холодильников")
OnDeph()
OnCond()
time.sleep(2)
OffAll()

#############Параметры колонны#####################################
#Температура срабатывания клапана холодильника
Tcond=29.0

#Температура дефлегматора стабилизации
Tst=45.0

#Температура дефлегматора отбора голов (уменьшил на 0.3 град.)
Thead=51.0

#Температура дефлегматора отбора тела
Tbody=63.5

#Температура низа колонны завершения перегона (было 85)
Tend=84.5

#Корректировка температурной установки дефлегматора
#(увеличение флегмового числа от температуры куба
#Tdeph=Tconst1-Kdeph*(Tниз-Tconst2)
#Коэффициент корректирования Tдеф (формула в тексте)
Kdeph=1.275862
#Базовая температура дефлегматора
Tconst1=63.5
#Базовая температура низа колонны (при АС=84% T=68.9 град C)
Tconst2=69.0

#Вычисление необходимой мощности нагрева в послеразгонных
#режимах производится по формуле P=Pbase+Kp*(Tниз-Tconst2), где:
#Tниз  - температура низа колонны;
#Pbase - мощность при максимуме спирта и отсутствии захлёба;
Pbase=18
#Kp    - коффициент роста мощности;
Kp=0.796460177
#Базовая мощность

#Таймаут ожидания завершения измерения (сек)
timeout=10

#Продолжительность стабилизации в сек
timeStab=5*60.0   #5 минут

#Продолжительность отбора голов в сек
timeHead=60*60.0  #60 минут

#Места и порядок расположения термометров:
locations=[u"Куб",u"1/2",u"Верх",u"Деф",u"Конд"]

#Зашитное время от отключения воды
TimeSafe=3*60.0   #3 минуты

#Хвосты идут примерно при ??? градусах цельсия низа колонны
#теперь это вопрос
##################################################################

#Разрешитель работы автомата
AutoEn=True


def DisplayT(stage):
    Temp=stage+": "
    for dt in locations:
        db_cur.execute("SELECT Name,T FROM DS18B20cur WHERE Name='%s'"%dt)
        dt=db_cur.fetchone()
        Temp+=(u"%s=%3.1f" % (dt[0],round(float(dt[1]),1)))+"  "
    Temp+=(u"P=%s"%Power.P)
    Temp+=(u"  Tдеф=%3.2f" % Tdeph)
    print(Temp)


def Auto():
    """Автомат, реализующий процесс регулирования перегона
    нужно только параметры перегона задавать"""
    global AutoEn
    #Cоединение с базой данных
    db_con=sqlite3.connect("Distiller.db")
    #Создание курсора БД
    db_cur=db_con.cursor()

    #Запуск процесса сбора температурных данных
    DS18B20db.StartMeasure()
    DS18B20db.StartWrite()
    #Запуск преобразования Брезенхема
    Power.On()

    #Зашитное время от отключения воды
    TimeS=time.time()

    while AutoEn:
        #Нужно подождать завершение измерения температуры
        DS18B20db.ReadyData.wait(timeout)
        timeB=time.time()
        DS18B20db.ReadyData.wait(timeout)
        if (time.time()-timeB) >= timeout:
            print("!!!ОШИБКА!!!: Сбой процесса измерения температур.")
            break
         
        #Чтение термометров из БД
        T=[]
        for dt in locations:
            db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
            T.append(float(db_cur.fetchone()[0]))

        #Если температура на выходе холодильника-конденсатора >
        #установленной, включаем охлаждение. Иначе отключаем.
        if T[4]>Tcond:
            OnCond()
        else:
            OffCond()

        #Если температура дефлегматора >
        #установленной, включаем охлаждение. Иначе отключаем.
        if T[3]>Tdeph:
            OnDeph()
            if TimeSafe<(time.time()-TimeS):
                AutoEn=False
                print("Превышен интервал ожидания охлаждения")
        else:
            OffDeph()
            TimeS=time.time()

    AutoEn=False
    #Отключаем нарев
    Power.Off()
    #Отключаем всё
    OffAll()
    #Останавливаем запись температур в БД
    DS18B20db.StopWrite()
    DS18B20db.StopMeasure()

#Температура дефлегматора стабилизации колонны
Tdeph=Tst

#Время старта
timeStart=datetime.datetime.now()

#Запускаем автомат в отдельном потоке
thrdAuto=threading.Thread(target=Auto)
thrdAuto.start()
print("Разгон")
#Устанавливаем мощность на 100%
Power.Set(100)

#Cоединение с базой данных
db_con=sqlite3.connect("Distiller.db")
#Создание курсора БД
db_cur=db_con.cursor()

#Ожидаем закипание
while AutoEn:
    DS18B20db.ReadyData.wait()
    DisplayT(u"Разгон")
    db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='1/2'")
    T,=db_cur.fetchone()
    if T>47.0:
        break
print("Стабилизация колонны")

#Фиксируем время начала закипания
timeBegin=time.time()
#Ждем время, отведенное на стабилизацию колонны
while time.time()-timeBegin<timeStab and AutoEn:
    DS18B20db.ReadyData.wait()
    DisplayT(u"Стабилизация")
    #Чтение термометров из БД
    T=[]
    for dt in locations:
        db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
        T.append(float(db_cur.fetchone()[0]))
    #Корректируем мощность
    P=Pbase+Kp*(T[0]-Tconst2)
    Power.Set(P)


print("Отбор голов")

#Температура дефлегматора отбора голов
Tdeph=Thead
#Фиксируем время начала отбора голов
timeBegin=time.time()
#Ждем время, отведенное на отбор голов
while time.time()-timeBegin<timeHead and AutoEn:
    DS18B20db.ReadyData.wait()
    DisplayT(u"Головы")
    #Чтение термометров из БД
    T=[]
    for dt in locations:
        db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
        T.append(float(db_cur.fetchone()[0]))
    #Корректируем мощность
    P=Pbase+Kp*(T[0]-Tconst2)
    Power.Set(P)

if AutoEn:
    End=raw_input("Поменяйте приемную емкость и нажмите Enter для запуска отбора тела")

#Температура дефлегматора отбора тела
Tdeph=Tbody

if AutoEn:
    print("Отбор тела")

#Отбор тела
while AutoEn:
    DS18B20db.ReadyData.wait()
    #Чтение термометров из БД
    T=[]
    for dt in locations:
        db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
        T.append(float(db_cur.fetchone()[0]))

    #Корректируем температуру включения дефлегматора
    #Tdeph=Tconst1-Kdeph*(Tниз-Tconst2)
    Tdeph=Tconst1-Kdeph*(T[0]-Tconst2)
    #Корректируем мощность
    P=Pbase+Kp*(T[0]-Tconst2)
    Power.Set(P)
    #Выводим показания
    DisplayT(u"Тело")
    #Если температура низа колонны превысила максимальную, завершаем перегон
    if T[0]>Tend:
        AutoEn=False

print("Процесс перегона завершен")

duration=relativedelta(datetime.datetime.now(), timeStart)
durstr='' if duration.days==0 else (str(duration.days)+' дней, ')
durstr+='' if (duration.days==0 and duration.hours==0) else (str(duration.hours)+' часов, ')
durstr+='' if (duration.days==0 and duration.hours==0 and duration.minutes==0) else (str(duration.minutes)+' минут.')
print("Продолжительность работы:"+durstr)


