#!/usr/bin/env python
#coding=utf8
"""
$Id: DS18B20.__init__.py,v 1.2 2014/05/11 10:54:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Эта программа находит все температурные датчики на шине 1-wire
и периодически выводит значения измеренных ими температур.
Также выводится время, затраченное на измерение температуры.
Исполнена в виде модуля для встраивания в проекты python.

Условия работы программы:
    должны быть установлены пакеты:
        owserver
        python-ow
    должен быть запущен owserver
    в файле /etc/owfs.conf должна быть добавлена строка:
        'server: device = /dev/ttyUSB0' (для USB9097)
        (или для DS9490 раскомментировать строку 'server: usb = all' )
"""

# подключение необходимых библиотек
import time, ow

#Минимальное время измерения температуры
MinMeasureTime=1.0

# инициализация ow - указание адреса и порта owserver (см. /etc/owfs.conf)
ow.init('localhost:4304')
#print ('инициализация ow на порт localhost:4304')

# создание объекта, отражающего сеть 1-wire
__onewire__=ow.Sensor('/')
#print ('создание объекта Sensor')

# отключение кэширования owserver
__onewire__.useCache(False)
#print ('включение кэширования owserver')

# получение списка цифровых термометров DS18B20 на шине 1-wire
__sensors__=__onewire__.find(type='DS18B20')
#print ('получение списка цифровых термометров DS18B20 на шине 1-wire')

# получение списка термометров DS18B20
__list__=[]
for ts in __sensors__:
    __list__.append('/'+ts.family+'.'+ts.id)
#print ('получение списка термометров')

def list():
    """Возвращает список цифровых термометров DS18B20 на шине 1-wire"""
    return __list__

#Получение значений температур DS18B20 (список кортежей)
def temperatures():
    """Возвращает измеренные температуры цифровых термометров DS18B20 на шине 1-wire
    в виде генератора списка кортежей (ID термометра, температура)"""
    # запуск температурного преобразования всех термометров
    ow._put('/simultaneous/temperature','1')
    #Ждем завершение измерения
    time.sleep(MinMeasureTime)

    #Опрос и выдача показаний термометров DS18B20
    for termometr in list():
        yield (termometr,float(ow._get(termometr+'/temperature')))

def main():
    """Тестовый опрос термометров и вывод результатов измерения"""
    timeSumm,i=0,0
    #бесконечный цикл измерения температуры и времени ее измерения (прерывание Ctl+C)
    while 1:
        #фиксация времени начала измерения
        timeStart=time.time()
        for temp in temperatures():
            thetmometer, temperature = temp
            print ('Термометр %s   Температура = %3.1fград.C' % (thetmometer, round(temperature,1)))

        timeCurr=time.time()-timeStart
        print ('Продолжительность измерения=%s сек' % timeCurr)
        timeSumm+=timeCurr
        i+=1
        print ('Среднее время измерения=%f сек' % (timeSumm/i))
        print ('')

#Если скрипт запущен автономно, запустить процесс измерения
if __name__=='__main__':
    main()
