#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
$Id: DS18B20autolocation.py,v 1.2 2014/05/31 11:52:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Модуль автоматически определяет и сохраняет в базе данных
места расположения цифровых термометров DS18B20

Нужно аппаратуру заправить водой и запустить данный модуль.
По завершении его работы в БД будут сохранены места расположения
цифровых термометров DS18B20 (в таблице DS18B20cur).

"""

import DS18B20db
import actuators, sqlite3, time, Power
from actuators import *

#Места расположения термометров:
locations=[u"Куб",u"1/2",u"Верх",u"Деф",u"Конд"]

def WaitBoiling(Tlist):
    """Ожидание условия закипания"""
    #Cоединение с базой данных
    cnct=sqlite3.connect("Distiller.db")
    #Создание курсора БД
    crsr=cnct.cursor()
    Vmax=0.0            #Скорость изменения температуры
    IDmax=u""           #ID термометра с макимальным ростом температуры
    while Vmax<1:       #Пока скорость изменения температуры менее 1град в сек
        #Нужно подождать результаты измерения температуры
        DS18B20db.ReadyData.wait()

        for ID in Tlist:    #Для каждого термометра из списка
            #Читаем два последних значения
            execute=u"SELECT ["+ID+u"] FROM DS18B20 ORDER BY DateTime DESC LIMIT 2"
            crsr.execute(execute)
            Ts=crsr.fetchall()
            if len(Ts)<2:
                break
            Tup=float(Ts[0][0]) #Самое свежее значение температуры
            Tlo=float(Ts[1][0]) #Предыдущее значение
            if (Tup-Tlo)>Vmax:  #Если разность превышает максимальну скорость
                Vmax=Tup-Tlo    #Новое значение скорости
                IDmax=ID        #Термометр
                print(u"ID=%s, Vmax=%s"%(IDmax,Vmax))
                
    crsr.execute(u"SELECT DateTime FROM DS18B20 ORDER BY DateTime DESC LIMIT 1")
    return (IDmax,Vmax,crsr.fetchone()[0])

#Отключение всех реле
OffAll()
#Заполнение холодильников
print("Заполнение холодильников")
OnCond()
OnDeph()
time.sleep(2)
OffAll()

#Cоединение с базой данных
db_con=sqlite3.connect("Distiller.db")
#Создание курсора БД
db_cur=db_con.cursor()
#Проверка списка термометров
db_cur.execute("SELECT * FROM DS18B20cur")
Tlist=[]     #Список термометров
for i in db_cur.fetchall():
    ID=i[0]
    Tlist.append(ID)
if len(Tlist)==0:
    End=raw_input("Цифровые термометры DS18B20 не обнаружены!!!")
    exit()    

#Запуск процесса измерения
DS18B20db.StartMeasure()
#Запуск процесса сбора температурных данных
DS18B20db.StartWrite()

#Установка полной мощности
Power.Set(100)

#Запуск преобразования Брезенхема
Power.On()
print("100% мощность нагрева")

#Ждем нагрева до закипания (как правило, термометр 1/2)
Boiling=WaitBoiling(Tlist)
print(u"%s Vmax=%s %s"%Boiling)
Power.Set(40)
print("Мощность снижена до 40%")

#Исключаем первый среагировавший термометр
Tlist.remove(Boiling[0])

#Ждем реакции следующего термометра (скорее верх колонны)
Boiling=WaitBoiling(Tlist)
print(u"%s Vmax=%s %s"%Boiling)

###Исключаем второй среагировавший термометр
##Tlist.remove(Boiling[0])
##
###Ждем реакции третьего термометра (наверное дефлегматор)
##Boiling=WaitBoiling(Tlist)
##print(u"%s Vmax=%s %s"%Boiling)


#Уменьшаем мощность до 30%
Power.Set(30)
print("Мощность снижена до 30%")
#Включаем клапаны
OnDeph()
OnCond()
print("Клапаны дефлегматора и конденсатора включены")
print("Стабилизация 40 сек")
#Стабилизация
time.sleep(40)
OffDeph()
print("Клапан дефлегматора отключен")
print("Выдержка 40 сек")
time.sleep(40)

#Запрашиваем из БД ID термометров в порядке убывания температур
db_cur.execute("SELECT ID FROM DS18B20cur ORDER BY T DESC")

i=0
#Присваиваем названия в порядке убывания температур
for dt in db_cur.fetchall():
    ID,=dt
    db_cur.execute("UPDATE DS18B20cur SET Name=? WHERE ID=?", (locations[i],ID))
    #Сохраняем БД
    db_con.commit()
    i+=1
#Отключаем нарев
Power.Off()
#Включаем охлаждение дефлегматора
OnDeph()
time.sleep(2)
#Отключаем всё
OffAll()

print ("Автоопределение мест подключения термометров завершено, данные сохранены в БД.")
db_cur.execute("SELECT * FROM DS18B20cur ORDER BY T DESC")
for dt in db_cur:
    print(u"%s %s %s"%dt)

#Останавливаем процесс сбора температурных данных
DS18B20db.StopWrite()
#Останавливаем измерение температур
DS18B20db.StopMeasure()

#print db_cur.fetchall()
#Закрываем соединение с БД
#db_con.close()

