#!/usr/bin/env python
#coding=utf8
"""
$Id: Power.py,v 1.2 2014/05/29 11:52:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Модуль обеспечивает регулировку уровня выдаваемый на ТЭНы
мощности по алгоритму Брезенхема.

Для работы программы должны быть установлены пакеты:
    gpio
"""

import actuators, time, threading
from actuators import *

step=1.0    #Шаг установки мощности = 1%
P=0         #Текущая мощность (0...100%)
period=50.0    #Длительность преобразования Брезенхема (сек)
#Разрешитель работы потока преобразования Брезенхема
Convert=False

def On():
    """Стартует поток регулятора мощности"""
    global Convert
    Convert=True
    thrdPwr=threading.Thread(target=Power)
    thrdPwr.start()

def Off():
    """Останавливает поток регулятора"""
    global Convert
    Convert=False

def Set(Pwr):
    """Установка мощности"""
    global P
    P=Pwr
    if P>100:
        P=100
    if P<0:
        P=0
    P=int((int((P+step/2)/step))*step)

def Power():
    """Реализация алгоритма Брезенхема
    для регулирования мощности ТЭНа"""
    while Convert:
        Pmax=int((100/step))    #Приведенная максимальная мощность
        Pwr=int(P/step)
        ErrP=Pmax-P
        for x in range(Pmax):
            if ErrP<Pmax/2:
                ErrP+=Pmax
                OnTEH1()
                OnTEH2()
            else:
                OffTEH1()
                OffTEH2()
            ErrP-=P
            time.sleep(period*step/100)
        OffTEH1()
        OffTEH2()

def main():
    """Ручная установка мощности и тестовое преобразование"""
    On()
    while True:
        End=raw_input("Значение мощности (0...100) или Enter для останова:")
        if End=="":
            Off()
            break
        try:
            Set(int(End))
            print("P=%s"%P)
        except ValueError:
            pass

if __name__=="__main__":
    try:
        main()
    finally:
        Off()
