#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
$Id: Steaming.py,v 1.2 2014/07/11 11:52:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Программа пропаривания колонны

Алгоритм:
1. Разгон (нагрев до закипания)
2. Работа в режиме "на себя" (30 минут)
3. Работа в режиме разделения (1 час)

"""

import actuators, sqlite3, time, DS18B20db, Power
from actuators import *

#############Параметры колонны#####################################
#Температура срабатывания клапана конденсатора
Tcond=30.0

#Маршевая мощность
PowerS=30

#Температура срабатывания клапана дефлегматора при работе на себя
TdephSelf=63.0

#Температура срабатывания клапана дефлегматора при промывке
TdephWashing=67.5

#Температура срабатывания клапана дефлегматора
Tdeph=66.5

#Время пропаривания (мин)
timeS=30

#Время промывки (мин)
timeW=60.0

#Места и порядок расположения термометров:
locations=[u"Куб",u"1/2",u"Верх",u"Деф",u"Конд"]

##################################################################

def WaitBoiling():
    """Ожидание закипания"""
    #Cоединение с базой данных
    cnct=sqlite3.connect("Distiller.db")
    #Создание курсора БД
    crsr=cnct.cursor()
    Vmax=0.0            #Скорость изменения температуры
    IDmax=u""           #ID термометра с макимальным ростом температуры
    while Vmax<1:       #Пока скорость изменения температуры менее 1град в сек
        #Нужно подождать результаты измерения температуры
        DS18B20db.ReadyData.wait()
        crsr.execute("SELECT ID FROM DS18B20cur WHERE Name='1/2'")
        ID,=crsr.fetchone()
        #Читаем два последних значения
        execute=u"SELECT ["+ID+u"] FROM DS18B20 ORDER BY DateTime DESC LIMIT 2"
        crsr.execute(execute)
        Ts=crsr.fetchall()
        if len(Ts)>=2:
            Tup=float(Ts[0][0]) #Самое свежее значение температуры
            Tlo=float(Ts[1][0]) #Предыдущее значение
            if (Tup-Tlo)>Vmax:  #Если разность превышает максимальну скорость
                Vmax=Tup-Tlo    #Новое значение скорости
        printT()
    crsr.execute(u"SELECT DateTime FROM DS18B20 ORDER BY DateTime DESC LIMIT 1")
    return (Vmax,crsr.fetchone()[0])

def printT():
    """Вывод показаний термометров"""
    #Чтение термометров из БД
    Temp=""
    for dt in locations:
        db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
        Temp+=(u"%s=%3.1f" % (dt,round(float(db_cur.fetchone()[0]),1)))+"  "
    Temp+=("P=%s"%Power.P)
    print(Temp)

def damper():
    #Нужно подождать завершение измерения температуры
    DS18B20db.ReadyData.wait()

    #Чтение термометров из БД
    T=[]
    for dt in locations:
        db_cur.execute("SELECT T FROM DS18B20cur WHERE Name='%s'"%dt)
        T.append(float(db_cur.fetchone()[0]))

    #Если температура на выходе холодильника-конденсатора>Tcond,
    #включаем охлаждение. Иначе отключаем.
    if T[4]>Tcond:
        OnCond()
    else:
        OffCond()

    #Если температура на выходе дефлегматора>Tbody,
    #включаем охлаждение. Иначе отключаем.
    if T[3]>Tdeph:
        OnDeph()
    else:
        OffDeph()

    #Вывод показаний
    printT()

#Отключение всех реле
OffAll()
print("Отключение всех реле")

#Заполнение холодильников
print("Заполнение холодильников")
OnDeph()
OnCond()
time.sleep(2)
OffAll()

#Cоединение с базой данных
db_con=sqlite3.connect("Distiller.db")
#Создание курсора БД
db_cur=db_con.cursor()
#Проверка списка термометров
db_cur.execute("SELECT * FROM DS18B20cur")
Tlist=[]     #Список термометров
for i in db_cur.fetchall():
    ID=i[0]
    Tlist.append(ID)
    if i[1]==None:
        End=raw_input("Требуется калибровка!!!")
        exit()    
        
if len(Tlist)==0:
    End=raw_input("Цифровые термометры DS18B20 не обнаружены!!!")
    exit()    

#Запуск процесса измерения температур
DS18B20db.StartMeasure()
#Запуск записи температур в БД
DS18B20db.StartWrite()

#Установка полной мощности
Power.Set(100)
#Запуск преобразования Брезенхема
Power.On()
print("Мощность 100%")

#Ждем нагрева до закипания (как правило, термометр 1/2)
print("Ожидание закипания")
Boiling=WaitBoiling()
#print(u"%s Vmax=%s %s"%Boiling)
Power.Set(PowerS)
print("Мощность снижена до %s"%PowerS)

#Работа на себя
Tdeph=TdephSelf
timeB=time.time()
print("Запуск цикла пропаривания")
while (time.time()-timeB)<(timeS*60):
    damper()

#Промывка
Tdeph=TdephWashing
timeB=time.time()
print("Запуск цикла промывки")
while (time.time()-timeB)<(timeW*60):
    damper()

print("Пропаривание завершено")
#Отключаем нарев
Power.Off()
print("Регулятор мощности остановлен")

#Продувка
OnCond()
OnDeph()
time.sleep(2)

#Отключаем всё
OffAll()
print("Все реле отключены")
#Останавливаем запись температур в БД
DS18B20db.StopWrite()
DS18B20db.StopMeasure()

print("Процесс сбора температур остановлен")

