#!/usr/bin/env python
#coding=utf8
"""
$Id: actuators.py,v 1.2 2014/05/12 12:54:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Модуль управления включением/отключением 2-х ТЭН и
2-х электроклапанов.

Условия работы программы:
    должен быть доступен пакет:
        gpio
"""
import gpio

#Назначаем выводы
pins={"TEH1" : "gpio2", "TEH2" : "gpio4", "Deph" : "gpio8", "Cond" : "gpio7"}

def OffAll():
    """Отключение всех ТЭН и клапанов"""
    for pin in pins.values():
        gpio.pinMode(pin,gpio.OUTPUT)
        gpio.digitalWrite(pin,gpio.LOW)

def OnAll():
    """Включение всех ТЭН и клапанов"""
    for pin in pins.values():
        gpio.pinMode(pin,gpio.OUTPUT)
        gpio.digitalWrite(pin,gpio.HIGH)

def OnTEH1():
    """Включение ТЭН1"""
    gpio.pinMode(pins['TEH1'],gpio.OUTPUT)
    gpio.digitalWrite(pins['TEH1'],gpio.HIGH)

def OnTEH2():
    """Включение ТЭН2"""
    gpio.pinMode(pins['TEH2'],gpio.OUTPUT)
    gpio.digitalWrite(pins['TEH2'],gpio.HIGH)

def OnDeph():
    """Включение электроклапана дефлегматора"""
    gpio.pinMode(pins['Deph'],gpio.OUTPUT)
    gpio.digitalWrite(pins['Deph'],gpio.HIGH)

def OnCond():
    """Включение электроклапана дефлегматора"""
    gpio.pinMode(pins['Cond'],gpio.OUTPUT)
    gpio.digitalWrite(pins['Cond'],gpio.HIGH)

def OffTEH1():
    """Выключение ТЭН1"""
    gpio.pinMode(pins['TEH1'],gpio.OUTPUT)
    gpio.digitalWrite(pins['TEH1'],gpio.LOW)

def OffTEH2():
    """Выключение ТЭН2"""
    gpio.pinMode(pins['TEH2'],gpio.OUTPUT)
    gpio.digitalWrite(pins['TEH2'],gpio.LOW)

def OffDeph():
    """Выключение электроклапана дефлегматора"""
    gpio.pinMode(pins['Deph'],gpio.OUTPUT)
    gpio.digitalWrite(pins['Deph'],gpio.LOW)

def OffCond():
    """Выключение электроклапана дефлегматора"""
    gpio.pinMode(pins['Cond'],gpio.OUTPUT)
    gpio.digitalWrite(pins['Cond'],gpio.LOW)

def main():
    """Тестовое включение/отключение"""
    import time
    print ("Для прерывания Ctrl+C")
    while(True):
        print("Включено всё")
        OnAll()
        time.sleep (0.2)
        print("Выключено всё")
        OffAll()
        time.sleep(0.8)
        print("Дефлегматор")
        OnDeph()
        time.sleep(0.2)
        print("Конденсатор")
        OnCond()
        time.sleep(0.2)
        print("ТЭН1")
        OnTEH1()
        time.sleep(0.2)
        print("ТЭН2")
        OnTEH2()
        time.sleep(0.2)
        print("Откл. дефлегматор")
        OffDeph()
        time.sleep(0.2)
        print("Откл. конденсатор")
        OffCond()
        time.sleep(0.2)
        print("Откл. ТЭН1")
        OffTEH1()
        time.sleep(0.2)
        print("Откл. ТЭН2")
        OffTEH2()
        time.sleep(0.2)

if __name__=="__main__":
    try:
        main()
    finally:
        OffAll()
    



